/**
 * @name Find uses of EFI variables
 * @description Finds all uses of EFI variables in a C/C++ codebase.
 * @kind problem
 * @problem.severity warning
 * @id cpp/find-uses-of-efi-variables
 * @tags security
 *       efi
 *       external interface
 * @precision low
 * @queryLanguages cpp
 */

import cpp
import semmle.code.cpp.security.FlowSources
import semmle.code.cpp.rangeanalysis.SimpleRangeAnalysis
import semmle.code.cpp.valuenumbering.GlobalValueNumbering

class EFIVarFunction extends Function {
  EFIVarFunction() {
    this.hasGlobalOrStdOrBslName("SmmSetVariable") or
    this.hasGlobalOrStdOrBslName("SmmGetVariable")
  }
}

class EFIVarPtrFunction extends PointerFieldAccess {
  EFIVarPtrFunction() {
    this.getTarget().hasName("SmmSetVariable") or
    this.getTarget().hasName("SmmGetVariable")
  }
}

predicate callHandlesEFIVariable(Expr e, Call c) {
  exists(Call x |
    // in case it is just a normal function call
    (
      x = c and
      x.getParent() = e and
      x.getTarget() instanceof EFIVarFunction
    )
  )
  or
  exists(VariableCall vc, PointerFieldAccess fa |
    // in case the function call is actually a pointer field access, the function name will be the name of the field itself
    c = vc and
    vc.getParent() = e and
    fa instanceof EFIVarPtrFunction and
    vc.getVariable() = fa.getTarget()
  )
}

class EFIVar extends Variable {
  EFIVar() {
    exists(Call c |
      callHandlesEFIVariable(_, c) and
      c.getAnArgument().(Access).getTarget() = this
    )
  }
}

from EFIVar v
select v, "This expression uses EFI variables"
